import logging
import liblynx

from starlette.authentication import (
    AuthenticationBackend,
    AuthenticationError,
    SimpleUser,
    AuthCredentials,
)
from starlette.datastructures import URL


class WAYFException(AuthenticationError):
    def __init__(self, redirect_to: str = None) -> None:
        self.redirect_to = redirect_to


class LibLynxAuthBackend(AuthenticationBackend):
    def __init__(self):
        self.connect = liblynx.Connect()

    async def authenticate(self, request):
        identification = self.connect.new_identification(
            request.client.host,
            str(URL(scope=request.scope)),
            request.headers.get("User-Agent", "<unknown>"),
        )
        logging.debug("identification in LibLynxAuthBackend is %s",
                      identification["id"])

        if identification["status"] == "identified":
            return (
                AuthCredentials(["authenticated"]),
                SimpleUser(identification["account"]["account_name"]),
            )
        elif identification["status"] == "wayf":
            logging.debug("WAYF needed, asking for %s",
                          identification["_links"]["wayf"]["href"])
            raise WAYFException(identification["_links"]["wayf"]["href"])



from setuptools import setup

from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read().strip()


setup(
    name="LibLynx",
    version="0.3",
    description="Python Library to interact with LibLynx",
    long_description_content_type="text/markdown",
    long_description=long_description,
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    author="Etienne Posthumus",
    author_email="posthumus@brill.com",
    url="https://gitlab.com/brillpublishers/code/liblynx",
    packages=["liblynx"],
    data_files=[('', ['LICENSE'])],
)



__version__ = "0.3"
import logging
import os
import time
from email.utils import parsedate
import requests


def make_url(path, sandbox=True):
    if sandbox:
        BASE = "https://sandbox.liblynx.com"
    else:
        BASE = "https://connect.liblynx.com"
    return "%s%s" % (BASE, path)


class APICAllException(Exception):
    pass


def if_success(arequest, msg, successcode=200):
    if arequest.status_code == successcode:
        logging.debug("%s worked" % msg)
        return arequest.json()
    else:
        raise APICAllException("%s failed. status: %s %s" %
                               (msg, arequest.status_code, arequest.text))


class Connect:
    def __init__(self, client_id=None, client_secret=None):
        client_id = client_id or os.environ.get("LIBLYNX_CLIENT_ID")
        if not client_id:
            raise Exception(
                "client_id not specified nor found from ENV LIBLYNX_CLIENT_ID")
        self.client_id = client_id
        client_secret = client_secret or os.environ.get(
            "LIBLYNX_CLIENT_SECRET")
        if not client_secret:
            raise Exception(
                "client_secret not specified nor found from ENV LIBLYNX_CLIENT_SECRET")
        self.client_secret = client_secret
        self.CACHE = {}

    def __getattr__(self, attr):
        value, expiry = self.CACHE.get(attr, (None, 0))
        if time.time() > expiry:
            getter = "_get_%s" % attr
            logging.debug("%s expired, trying getattr with %s" %
                          (attr, getter))
            getter = self.__getattribute__(getter)
            if getter:
                value, expiry = getter()
                logging.debug("got new value for %s expires at %s" %
                              (attr, time.ctime(expiry)))
                self.CACHE[attr] = (value, expiry)
        return value

    def _get_access_token(self):
        r = requests.post(make_url("/oauth/v2/token"),
                          {"grant_type": "client_credentials"}, auth=(self.client_id, self.client_secret))
        if r.status_code == 200:
            r_json = r.json()
            logging.debug("access_token acquired for %s", self.client_id)
            expiry = time.time() + (r_json["expires_in"] - 60)
            return r_json["access_token"], expiry
        logging.error("access_token failed %s %s", r.status_code, r.text)

    def _get_endpoint(self):
        logging.debug("api endpoint expired getting a new one")
        headers = {"Authorization": "Bearer %s" %
                   self.access_token, "Accept": "application/json"}
        r = requests.get(make_url("/api"), headers=headers)
        if r.status_code == 200:
            # Make it one hour less than the expiry
            expires = time.mktime(
                parsedate((r.headers.get("Expires")))) - (60*60)
            api = r.json()
            logging.debug("new api endpoint acquired")
            return api, expires
        else:
            raise APICAllException(
                "api endpoint failed %s %s" % (r.status_code, r.text))

    def new_identification(self, ip, url, user_agent):
        headers = {"Authorization": "Bearer %s" % self.access_token,
                   "Accept": "application/json", "Content-Type": "application/json"}
        data = {"ip": ip, "url": url, "user_agent": user_agent}
        r = requests.post(self.endpoint["_links"]["@new_identification"]
                          ["href"], json=data, headers=headers)
        return if_success(r, "new identification", successcode=201)

    def identification(self, anid):
        headers = {"Authorization": "Bearer %s" % self.access_token,
                   "Accept": "application/json", "Content-Type": "application/json"}
        r = requests.get(self.endpoint["_links"]["@get_identification"]
                         ["href"].replace("{id}", anid), headers=headers)
        return if_success(r, "get identification", successcode=200)
